@@ -10,13 +10,22 @@ let viewerState = {
   uiLanguage: 'en',
 };
 
+function dropUserLang(query) {
+  const q = new URLSearchParams(query);
+  q.delete('userlang');
+  const pre = (query.startsWith('?') && q.size != 0 ? '?' : '');
+  return pre + q.toString();
+}
+
 function userUrl2IframeUrl(url) {
   if ( url == '' ) {
     return blankPageUrl;
   }
 
   if ( url.startsWith('search?') ) {
-    return `${root}/${url}`;
+    const q = new URLSearchParams(url.slice("search?".length));
+    q.set('userlang', viewerState.uiLanguage);
+    return `${root}/search?${q.toString()}`;
   }
 
   return `${root}/content/${url}`;
@@ -73,7 +82,7 @@ function quasiUriEncode(s, specialSymbols) {
 function performSearch() {
   const searchbox = document.getElementById('kiwixsearchbox');
   const q = encodeURIComponent(searchbox.value);
-  gotoUrl(`/search?books.name=${currentBook}&pattern=${q}`);
+  gotoUrl(`/search?books.name=${currentBook}&pattern=${q}&userlang=${viewerState.uiLanguage}`);
 }
 
 function makeJSLink(jsCodeString, linkText, linkAttr="") {
@@ -148,7 +157,7 @@ function iframeUrl2UserUrl(url, query) {
   }
 
   if ( url == `${root}/search` ) {
-    return `search${query}`;
+    return `search${dropUserLang(query)}`;
   }
 
   url = url.slice(root.length);
@@ -249,6 +258,25 @@ function handle_location_hash_change() {
   history.replaceState(viewerState, null);
 }
 
+function translateErrorPageIfNeeded() {
+  const cw = contentIframe.contentWindow;
+  if ( cw.KIWIX_RESPONSE_TEMPLATE && cw.KIWIX_RESPONSE_DATA ) {
+    const template = htmlDecode(cw.KIWIX_RESPONSE_TEMPLATE);
+
+    // cw.KIWIX_RESPONSE_DATA belongs to the iframe context and running
+    // I18n.render() on it directly in the top context doesn't work correctly
+    // because the type checks (obj.__proto__ == ???.prototype) in
+    // I18n.instantiateParameterizedMessages() always fail (String.prototype
+    // refers to different objects in different contexts).
+    // Work arround that issue by copying the object into our context.
+    const params = JSON.parse(JSON.stringify(cw.KIWIX_RESPONSE_DATA));
+
+    const html = I18n.render(template, params);
+    const htmlDoc = new DOMParser().parseFromString(html, "text/html");
+    cw.document.documentElement.innerHTML = htmlDoc.documentElement.innerHTML;
+  }
+}
+
 function handle_content_url_change() {
   const iframeLocation = contentIframe.contentWindow.location;
   console.log('handle_content_url_change: ' + iframeLocation.href);
@@ -258,6 +286,7 @@ function handle_content_url_change() {
   const newHash = iframeUrl2UserUrl(iframeContentUrl, iframeContentQuery);
   history.replaceState(viewerState, null, makeURL(location.search, newHash));
   updateCurrentBookIfNeeded(newHash);
+  translateErrorPageIfNeeded();
 };
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -290,12 +319,23 @@ function isExternalUrl(url) {
       || url.startsWith("https:");
 }
 
+function getRealHref(target) {
+  // In case of wombat in the middle, wombat will rewrite the href value to the original url (external link)
+  // This is not what we want. Let's ask wombat to not rewrite href
+  const old_no_rewrite = target._no_rewrite;
+  target._no_rewrite = true;
+  const target_href = target.href;
+  target._no_rewrite = old_no_rewrite;
+  return target_href;
+}
+
 function onClickEvent(e) {
   const iframeDocument = contentIframe.contentDocument;
   const target = matchingAncestorElement(e.target, iframeDocument, "a");
   if (target !== null && "href" in target) {
-    if ( isExternalUrl(target.href) ) {
-      const possiblyBlockedLink = blockLink(target.href);
+    const target_href = getRealHref(target);
+    if (isExternalUrl(target_href)) {
+      const possiblyBlockedLink = blockLink(target_href);
       if ( e.ctrlKey || e.shiftKey ) {
         // The link will be loaded in a new tab/window - update the link
         // and let the browser handle the rest.
@@ -485,6 +525,7 @@ function changeUILanguage() {
   viewerState.uiLanguage = lang;
   setUserLanguage(lang, () => {
     updateUIText();
+    translateErrorPageIfNeeded();
     history.pushState(viewerState, null);
   });
 }
@@ -505,9 +546,8 @@ function setupViewer() {
   const lang = getUserLanguage();
   setUserLanguage(lang, finishViewerSetupOnceTranslationsAreLoaded);
   viewerState.uiLanguage = lang;
-  const q = new URLSearchParams(window.location.search);
-  q.delete('userlang');
-  const rewrittenURL = makeURL(q.toString(), location.hash);
+  const cleanedUpQuery = dropUserLang(window.location.search);
+  const rewrittenURL = makeURL(cleanedUpQuery, location.hash);
   history.replaceState(viewerState, null, rewrittenURL);
 
   kiwixToolBarWrapper.style.display = 'block';
