@@ -59,7 +59,7 @@ const ResourceCollection resources200Compressible{
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/autoComplete/css/autoComplete.css" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/autoComplete/css/autoComplete.css?cacheid=ef30cd42" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/i18n.js" },
-  { STATIC_CONTENT,  "/ROOT%23%3F/skin/i18n.js?cacheid=6a8c6fb2" },
+  { STATIC_CONTENT,  "/ROOT%23%3F/skin/i18n.js?cacheid=071abc9a" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/index.css" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/index.css?cacheid=1e78e7cf" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/index.js" },
@@ -75,7 +75,7 @@ const ResourceCollection resources200Compressible{
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/taskbar.css" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/taskbar.css?cacheid=e014a885" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/viewer.js" },
-  { STATIC_CONTENT,  "/ROOT%23%3F/skin/viewer.js?cacheid=201653b8" },
+  { STATIC_CONTENT,  "/ROOT%23%3F/skin/viewer.js?cacheid=5fc4badf" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/fonts/Poppins.ttf" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/fonts/Poppins.ttf?cacheid=af705837" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/fonts/Roboto.ttf" },
@@ -83,6 +83,8 @@ const ResourceCollection resources200Compressible{
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/i18n/test.json" },
   // TODO: implement cache management of i18n resources
   //{ STATIC_CONTENT, "/ROOT%23%3F/skin/i18n/test.json?cacheid=unknown" },
+  { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/languages.js" },
+  { STATIC_CONTENT, "/ROOT%23%3F/skin/languages.js?cacheid=5be77f5c" },
 
   { DYNAMIC_CONTENT, "/ROOT%23%3F/catalog/search" },
 
@@ -144,12 +146,12 @@ const ResourceCollection resources200Uncompressible{
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/hash.png?cacheid=f836e872" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/magnet.png" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/magnet.png?cacheid=73b6bddf" },
+  { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/polyfills.js" },
+  { STATIC_CONTENT,  "/ROOT%23%3F/skin/polyfills.js?cacheid=a0e0343d" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/search-icon.svg" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/search-icon.svg?cacheid=b10ae7ed" },
   { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/search_results.css" },
   { STATIC_CONTENT,  "/ROOT%23%3F/skin/search_results.css?cacheid=76d39c84" },
-  { DYNAMIC_CONTENT, "/ROOT%23%3F/skin/languages.js" },
-  { STATIC_CONTENT, "/ROOT%23%3F/skin/languages.js?cacheid=96f2cf73" },
 
   { ZIM_CONTENT,     "/ROOT%23%3F/raw/zimfile/meta/Title" },
   { ZIM_CONTENT,     "/ROOT%23%3F/raw/zimfile/meta/Description" },
@@ -285,8 +287,9 @@ R"EXPECTEDRESULT(      href="/ROOT%23%3F/skin/kiwix.css?cacheid=2158fad9"
     <link rel="mask-icon" href="/ROOT%23%3F/skin/favicon/safari-pinned-tab.svg?cacheid=8d487e95" color="#5bbad5">
     <link rel="shortcut icon" href="/ROOT%23%3F/skin/favicon/favicon.ico?cacheid=92663314">
     <meta name="msapplication-config" content="/ROOT%23%3F/skin/favicon/browserconfig.xml?cacheid=f29a7c4a">
-    <script type="module" src="/ROOT%23%3F/skin/i18n.js?cacheid=6a8c6fb2" defer></script>
-    <script type="text/javascript" src="/ROOT%23%3F/skin/languages.js?cacheid=96f2cf73" defer></script>
+    <script type="text/javascript" src="./skin/polyfills.js?cacheid=a0e0343d"></script>
+    <script type="module" src="/ROOT%23%3F/skin/i18n.js?cacheid=071abc9a" defer></script>
+    <script type="text/javascript" src="/ROOT%23%3F/skin/languages.js?cacheid=5be77f5c" defer></script>
     <script src="/ROOT%23%3F/skin/isotope.pkgd.min.js?cacheid=2e48d392" defer></script>
     <script src="/ROOT%23%3F/skin/iso6391To3.js?cacheid=ecde2bb3"></script>
     <script type="text/javascript" src="/ROOT%23%3F/skin/index.js?cacheid=ce19da2a" defer></script>
@@ -318,9 +321,10 @@ R"EXPECTEDRESULT(                                <img src="${root}/skin/download
 R"EXPECTEDRESULT(    <link type="text/css" href="./skin/kiwix.css?cacheid=2158fad9" rel="Stylesheet" />
     <link type="text/css" href="./skin/taskbar.css?cacheid=e014a885" rel="Stylesheet" />
     <link type="text/css" href="./skin/autoComplete/css/autoComplete.css?cacheid=ef30cd42" rel="Stylesheet" />
-    <script type="module" src="./skin/i18n.js?cacheid=6a8c6fb2" defer></script>
-    <script type="text/javascript" src="./skin/languages.js?cacheid=96f2cf73" defer></script>
-    <script type="text/javascript" src="./skin/viewer.js?cacheid=201653b8" defer></script>
+    <script type="text/javascript" src="./skin/polyfills.js?cacheid=a0e0343d"></script>
+    <script type="module" src="./skin/i18n.js?cacheid=071abc9a" defer></script>
+    <script type="text/javascript" src="./skin/languages.js?cacheid=5be77f5c" defer></script>
+    <script type="text/javascript" src="./skin/viewer.js?cacheid=5fc4badf" defer></script>
     <script type="text/javascript" src="./skin/autoComplete/autoComplete.min.js?cacheid=1191aaaf"></script>
       const blankPageUrl = root + "/skin/blank.html?cacheid=6b1fa032";
           <label for="kiwix_button_show_toggle"><img src="./skin/caret.png?cacheid=22b942b4" alt=""></label>
@@ -337,6 +341,7 @@ R"EXPECTEDRESULT(    <link type="text/css" href="./skin/kiwix.css?cacheid=2158fa
       // a page rendered from static/templates/no_search_result_html
       /* url */ "/ROOT%23%3F/search?content=poor&pattern=whatever",
 R"EXPECTEDRESULT(    <link type="text/css" href="/ROOT%23%3F/skin/search_results.css?cacheid=76d39c84" rel="Stylesheet" />
+      window.KIWIX_RESPONSE_DATA = { "CSS_URL" : "/ROOT%23%3F/skin/search_results.css?cacheid=76d39c84", "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "fulltext-search-unavailable", "params" : { } }, "details" : [ { "p" : { "msgid" : "no-search-results", "params" : { } } } ] };
 )EXPECTEDRESULT"
     },
   };
@@ -535,6 +540,7 @@ struct ExpectedResponseData
 {
   const std::string expectedPageTitle;
   const std::string expectedCssUrl;
+  const std::string expectedKiwixResponseData;
   const std::string bookName;
   const std::string bookTitle;
   const std::string expectedBody;
@@ -544,6 +550,7 @@ enum ExpectedResponseDataType
 {
   expected_page_title,
   expected_css_url,
+  expected_kiwix_response_data,
   book_name,
   book_title,
   expected_body
@@ -556,11 +563,13 @@ ExpectedResponseData operator==(ExpectedResponseDataType t, std::string s)
 {
   switch (t)
   {
-    case expected_page_title: return ExpectedResponseData{s, "", "", "", ""};
-    case expected_css_url:    return ExpectedResponseData{"", s, "", "", ""};
-    case book_name:           return ExpectedResponseData{"", "", s, "", ""};
-    case book_title:          return ExpectedResponseData{"", "", "", s, ""};
-    case expected_body:       return ExpectedResponseData{"", "", "", "", s};
+    case expected_page_title: return ExpectedResponseData{s, "", "", "", "", ""};
+    case expected_css_url:    return ExpectedResponseData{"", s, "", "", "", ""};
+    case expected_kiwix_response_data:
+                              return ExpectedResponseData{"", "", s, "", "", ""};
+    case book_name:           return ExpectedResponseData{"", "", "", s, "", ""};
+    case book_title:          return ExpectedResponseData{"", "", "", "", s, ""};
+    case expected_body:       return ExpectedResponseData{"", "", "", "", "", s};
     default: assert(false); return ExpectedResponseData{};
   }
 }
@@ -579,6 +588,7 @@ ExpectedResponseData operator&&(const ExpectedResponseData& a,
   return ExpectedResponseData{
     selectNonEmpty(a.expectedPageTitle, b.expectedPageTitle),
     selectNonEmpty(a.expectedCssUrl, b.expectedCssUrl),
+    selectNonEmpty(a.expectedKiwixResponseData, b.expectedKiwixResponseData),
     selectNonEmpty(a.bookName, b.bookName),
     selectNonEmpty(a.bookTitle, b.bookTitle),
     selectNonEmpty(a.expectedBody, b.expectedBody)
@@ -607,19 +617,29 @@ private:
 std::string TestContentIn404HtmlResponse::expectedResponse() const
 {
   const std::string frag[] =  {
+    // frag[0]
     R"FRAG(<!DOCTYPE html>
 <html xmlns="http://www.w3.org/1999/xhtml">
   <head>
     <meta content="text/html;charset=UTF-8" http-equiv="content-type" />
     <title>)FRAG",
 
+    // frag[1]
     R"FRAG(</title>
 )FRAG",
 
-    R"FRAG(
+    // frag[2]
+    R"(    <script>
+      window.KIWIX_RESPONSE_TEMPLATE = )" + ERROR_HTML_TEMPLATE_JS_STRING + R"(;
+      window.KIWIX_RESPONSE_DATA = )",
+
+    // frag[3]
+    R"FRAG(;
+    </script>
   </head>
   <body>)FRAG",
 
+  // frag[4]
   R"FRAG(  </body>
 </html>
 )FRAG"
@@ -630,8 +650,10 @@ std::string TestContentIn404HtmlResponse::expectedResponse() const
        + frag[1]
        + pageCssLink()
        + frag[2]
+       + expectedKiwixResponseData
+       + frag[3]
        + expectedBody
-       + frag[3];
+       + frag[4];
 }
 
 std::string TestContentIn404HtmlResponse::pageTitle() const
@@ -648,7 +670,8 @@ std::string TestContentIn404HtmlResponse::pageCssLink() const
 
   return R"(    <link type="text/css" href=")"
        + expectedCssUrl
-       + R"(" rel="Stylesheet" />)";
+       + R"(" rel="Stylesheet" />)"
+       + "\n";
 }
 
 class TestContentIn400HtmlResponse : public TestContentIn404HtmlResponse
@@ -676,6 +699,7 @@ TEST_F(ServerTest, Http404HtmlError)
   using namespace TestingOfHtmlResponses;
   const std::vector<TestContentIn404HtmlResponse> testData{
     { /* url */ "/ROOT%23%3F/random?content=non-existent-book",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "non-existent-book" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -685,6 +709,7 @@ TEST_F(ServerTest, Http404HtmlError)
 
     { /* url */ "/ROOT%23%3F/random?content=non-existent-book&userlang=test",
       expected_page_title=="[I18N TESTING] Not Found - Try Again" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "non-existent-book" } } } ] })" &&
       expected_body==R"(
     <h1>[I18N TESTING] Content not found, but at least the server is alive</h1>
     <p>
@@ -693,6 +718,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/suggest?content=no-such-book&term=whatever",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "no-such-book" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -701,6 +727,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/catalog/",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/catalog/" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -710,6 +737,7 @@ TEST_F(ServerTest, Http404HtmlError)
 
     { /* url */ "/ROOT%23%3F/catalog/?userlang=test",
       expected_page_title=="[I18N TESTING] Not Found - Try Again" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/catalog/" } } } ] })" &&
       expected_body==R"(
     <h1>[I18N TESTING] Content not found, but at least the server is alive</h1>
     <p>
@@ -718,6 +746,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/catalog/invalid_endpoint",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/catalog/invalid_endpoint" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -727,6 +756,7 @@ TEST_F(ServerTest, Http404HtmlError)
 
     { /* url */ "/ROOT%23%3F/catalog/invalid_endpoint?userlang=test",
       expected_page_title=="[I18N TESTING] Not Found - Try Again" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/catalog/invalid_endpoint" } } } ] })" &&
       expected_body==R"(
     <h1>[I18N TESTING] Content not found, but at least the server is alive</h1>
     <p>
@@ -735,6 +765,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/content/invalid-book/whatever",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/invalid-book/whatever" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "whatever", "SEARCH_URL" : "/ROOT%23%3F/search?pattern=whatever" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -748,6 +779,7 @@ TEST_F(ServerTest, Http404HtmlError)
     { /* url */ "/ROOT%23%3F/content/zimfile/invalid-article",
       book_name=="zimfile" &&
       book_title=="Ray Charles" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/zimfile/invalid-article" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "invalid-article", "SEARCH_URL" : "/ROOT%23%3F/search?content=zimfile&pattern=invalid-article" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -759,6 +791,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ R"(/ROOT%23%3F/content/"><svg onload=alert(1)>)",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/\"><svg onload%3Dalert(1)>" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "\"><svg onload=alert(1)>", "SEARCH_URL" : "/ROOT%23%3F/search?pattern=%22%3E%3Csvg%20onload%3Dalert(1)%3E" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -772,6 +805,7 @@ TEST_F(ServerTest, Http404HtmlError)
     { /* url */ R"(/ROOT%23%3F/content/zimfile/"><svg onload=alert(1)>)",
       book_name=="zimfile" &&
       book_title=="Ray Charles" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/zimfile/\"><svg onload%3Dalert(1)>" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "\"><svg onload=alert(1)>", "SEARCH_URL" : "/ROOT%23%3F/search?content=zimfile&pattern=%22%3E%3Csvg%20onload%3Dalert(1)%3E" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -782,10 +816,27 @@ TEST_F(ServerTest, Http404HtmlError)
     </p>
 )"  },
 
+    // XXX: This test case is against a "</script>" string appearing inside
+    // XXX: javascript code that will confuse the HTML parser
+    { /* url */ R"(/ROOT%23%3F/content/zimfile/</script>)",
+      book_name=="zimfile" &&
+      book_title=="Ray Charles" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/zimfile/</scr\ipt>" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "script>", "SEARCH_URL" : "/ROOT%23%3F/search?content=zimfile&pattern=script%3E" } } } ] })" &&
+      expected_body==R"(
+    <h1>Not Found</h1>
+    <p>
+      The requested URL "/ROOT%23%3F/content/zimfile/&lt;/script&gt;" was not found on this server.
+    </p>
+    <p>
+      Make a full text search for <a href="/ROOT%23%3F/search?content=zimfile&pattern=script%3E">script&gt;</a>
+    </p>
+)"  },
+
     { /* url */ "/ROOT%23%3F/content/zimfile/invalid-article?userlang=test",
       expected_page_title=="[I18N TESTING] Not Found - Try Again" &&
       book_name=="zimfile" &&
       book_title=="Ray Charles" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/content/zimfile/invalid-article" } } }, { "p" : { "msgid" : "suggest-search", "params" : { "PATTERN" : "invalid-article", "SEARCH_URL" : "/ROOT%23%3F/search?content=zimfile&pattern=invalid-article" } } } ] })" &&
       expected_body==R"(
     <h1>[I18N TESTING] Content not found, but at least the server is alive</h1>
     <p>
@@ -797,6 +848,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/raw/no-such-book/meta/Title",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/raw/no-such-book/meta/Title" } } }, { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "no-such-book" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -808,6 +860,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/raw/zimfile/XYZ",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/raw/zimfile/XYZ" } } }, { "p" : { "msgid" : "invalid-raw-data-type", "params" : { "DATATYPE" : "XYZ" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -819,6 +872,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/raw/zimfile/meta/invalid-metadata",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/raw/zimfile/meta/invalid-metadata" } } }, { "p" : { "msgid" : "raw-entry-not-found", "params" : { "DATATYPE" : "meta", "ENTRY" : "invalid-metadata" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -830,6 +884,7 @@ TEST_F(ServerTest, Http404HtmlError)
 )"  },
 
     { /* url */ "/ROOT%23%3F/raw/zimfile/content/invalid-article",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "404-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "url-not-found", "params" : { "url" : "/ROOT%23%3F/raw/zimfile/content/invalid-article" } } }, { "p" : { "msgid" : "raw-entry-not-found", "params" : { "DATATYPE" : "content", "ENTRY" : "invalid-article" } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -845,6 +900,7 @@ TEST_F(ServerTest, Http404HtmlError)
       expected_css_url=="/ROOT%23%3F/skin/search_results.css?cacheid=76d39c84" &&
       book_name=="poor" &&
       book_title=="poor" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : "/ROOT%23%3F/skin/search_results.css?cacheid=76d39c84", "PAGE_HEADING" : { "msgid" : "404-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "fulltext-search-unavailable", "params" : { } }, "details" : [ { "p" : { "msgid" : "no-search-results", "params" : { } } } ] })" &&
       expected_body==R"(
     <h1>Not Found</h1>
     <p>
@@ -866,6 +922,7 @@ TEST_F(ServerTest, Http400HtmlError)
   using namespace TestingOfHtmlResponses;
   const std::vector<TestContentIn400HtmlResponse> testData{
     { /* url */ "/ROOT%23%3F/search",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search" } } }, { "p" : { "msgid" : "too-many-books", "params" : { "LIMIT" : "3", "NB_BOOKS" : "4" } } } ] })" &&
       expected_body== R"(
     <h1>Invalid request</h1>
     <p>
@@ -876,6 +933,7 @@ TEST_F(ServerTest, Http400HtmlError)
     </p>
 )"  },
     { /* url */ "/ROOT%23%3F/search?content=zimfile",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?content=zimfile" } } }, { "p" : { "msgid" : "no-query", "params" : { } } } ] })" &&
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
@@ -886,6 +944,7 @@ TEST_F(ServerTest, Http400HtmlError)
     </p>
 )"  },
     { /* url */ "/ROOT%23%3F/search?content=non-existing-book&pattern=asdfqwerty",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?content=non-existing-book&pattern=asdfqwerty" } } }, { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "non-existing-book" } } } ] })" &&
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
@@ -896,6 +955,7 @@ TEST_F(ServerTest, Http400HtmlError)
     </p>
 )"  },
     { /* url */ "/ROOT%23%3F/search?content=non-existing-book&pattern=a\"<script foo>",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?content=non-existing-book&pattern=a%22%3Cscript%20foo%3E" } } }, { "p" : { "msgid" : "no-such-book", "params" : { "BOOK_NAME" : "non-existing-book" } } } ] })" &&
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
@@ -908,6 +968,7 @@ TEST_F(ServerTest, Http400HtmlError)
     // There is a flaw in our way to handle query string, we cannot differenciate
     // between `pattern` and `pattern=`
     { /* url */ "/ROOT%23%3F/search?books.filter.lang=eng&pattern",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?books.filter.lang=eng&pattern" } } }, { "p" : { "msgid" : "no-query", "params" : { } } } ] })" &&
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
@@ -918,6 +979,7 @@ TEST_F(ServerTest, Http400HtmlError)
     </p>
 )"  },
     { /* url */ "/ROOT%23%3F/search?pattern=foo",
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?pattern=foo" } } }, { "p" : { "msgid" : "too-many-books", "params" : { "LIMIT" : "3", "NB_BOOKS" : "4" } } } ] })" &&
       expected_body==R"(
     <h1>Invalid request</h1>
     <p>
@@ -927,6 +989,20 @@ TEST_F(ServerTest, Http400HtmlError)
       Too many books requested (4) where limit is 3
     </p>
 )"  },
+
+    // Testing of translation
+    { /* url */ "/ROOT%23%3F/search?content=zimfile&userlang=test",
+      expected_page_title=="[I18N TESTING] Invalid request ($400 fine must be paid)" &&
+      expected_kiwix_response_data==R"({ "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "400-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "400-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "invalid-request", "params" : { "url" : "/ROOT%23%3F/search?content=zimfile&userlang=test" } } }, { "p" : { "msgid" : "no-query", "params" : { } } } ] })" &&
+      expected_body==R"(
+    <h1>[I18N TESTING] -400 karma for an invalid request</h1>
+    <p>
+      [I18N TESTING] Invalid URL: "/ROOT%23%3F/search?content=zimfile&userlang=test"
+    </p>
+    <p>
+      [I18N TESTING] Kiwix can read your thoughts but it is against GDPR. Please provide your query explicitly.
+    </p>
+)"  },
   };
 
   for ( const auto& t : testData ) {
@@ -1023,7 +1099,10 @@ TEST_F(ServerTest, 500)
   <head>
     <meta content="text/html;charset=UTF-8" http-equiv="content-type" />
     <title>Internal Server Error</title>
-
+    <script>
+      window.KIWIX_RESPONSE_TEMPLATE = )" + ERROR_HTML_TEMPLATE_JS_STRING + R"(;
+      window.KIWIX_RESPONSE_DATA = { "CSS_URL" : false, "PAGE_HEADING" : { "msgid" : "500-page-heading", "params" : { } }, "PAGE_TITLE" : { "msgid" : "500-page-title", "params" : { } }, "details" : [ { "p" : { "msgid" : "500-page-text", "params" : { } } }, { "p" : { "msgid" : "non-translated-text", "params" : { "MSG" : "Entry redirect_loop.html is a redirect entry." } } } ] };
+    </script>
   </head>
   <body>
     <h1>Internal Server Error</h1>
@@ -1041,6 +1120,7 @@ TEST_F(ServerTest, 500)
   const auto r = zfs1_->GET("/ROOT%23%3F/content/poor/A/redirect_loop.html");
   EXPECT_EQ(r->status, 500);
   EXPECT_EQ(r->body, expectedBody);
+  EXPECT_EQ(r->get_header_value("Content-Type"), "text/html; charset=utf-8");
   }
 }
 
@@ -1050,106 +1130,194 @@ TEST_F(ServerTest, UserLanguageList)
   EXPECT_EQ(r->body,
 R"EXPECTEDRESPONSE(const uiLanguages = [
   {
-    "الإنجليزية": "ar"
+    "iso_code": "ar",
+    "self_name": "الإنجليزية",
+    "translation_count": 25
   },
   {
-    "বাংলা": "bn"
+    "iso_code": "bn",
+    "self_name": "বাংলা",
+    "translation_count": 14
   },
   {
-    "Čeština": "cs"
+    "iso_code": "br",
+    "self_name": "brezhoneg",
+    "translation_count": 35
   },
   {
-    "Deutsch": "de"
+    "iso_code": "cs",
+    "self_name": "Čeština",
+    "translation_count": 25
   },
   {
-    "English": "en"
+    "iso_code": "dag",
+    "self_name": "Silimiinsili",
+    "translation_count": 24
   },
   {
-    "español": "es"
+    "iso_code": "de",
+    "self_name": "Deutsch",
+    "translation_count": 57
   },
   {
-    "suomi": "fi"
+    "iso_code": "en",
+    "self_name": "English",
+    "translation_count": 58
   },
   {
-    "Français": "fr"
+    "iso_code": "es",
+    "self_name": "español",
+    "translation_count": 48
   },
   {
-    "עברית": "he"
+    "iso_code": "fi",
+    "self_name": "suomi",
+    "translation_count": 22
   },
   {
-    "हिन्दी": "hi"
+    "iso_code": "fr",
+    "self_name": "Français",
+    "translation_count": 57
   },
   {
-    "Հայերեն": "hy"
+    "iso_code": "ha",
+    "self_name": "Turanci",
+    "translation_count": 57
   },
   {
-    "interlingua": "ia"
+    "iso_code": "he",
+    "self_name": "עברית",
+    "translation_count": 57
   },
   {
-    "italiano": "it"
+    "iso_code": "hi",
+    "self_name": "हिन्दी",
+    "translation_count": 49
   },
   {
-    "日本語": "ja"
+    "iso_code": "hy",
+    "self_name": "Հայերեն",
+    "translation_count": 15
   },
   {
-    "한국어": "ko"
+    "iso_code": "ia",
+    "self_name": "interlingua",
+    "translation_count": 49
   },
   {
-    "kurdî": "ku-latn"
+    "iso_code": "ig",
+    "self_name": "Bekee",
+    "translation_count": 57
   },
   {
-    "Lëtzebuergesch": "lb"
+    "iso_code": "it",
+    "self_name": "italiano",
+    "translation_count": 34
   },
   {
-    "македонски": "mk"
+    "iso_code": "ja",
+    "self_name": "日本語",
+    "translation_count": 26
   },
   {
-    "Bahasa Melayu": "ms"
+    "iso_code": "ko",
+    "self_name": "한국어",
+    "translation_count": 13
   },
   {
-    "Nederlands": "nl"
+    "iso_code": "ku-latn",
+    "self_name": "kurdî",
+    "translation_count": 26
   },
   {
-    "ߒߞߏ": "nqo"
+    "iso_code": "lb",
+    "self_name": "Lëtzebuergesch",
+    "translation_count": 22
   },
   {
-    "ଓଡ଼ିଆ": "or"
+    "iso_code": "mk",
+    "self_name": "македонски",
+    "translation_count": 57
   },
   {
-    "Polski": "pl"
+    "iso_code": "ms",
+    "self_name": "Bahasa Melayu",
+    "translation_count": 14
   },
   {
-    "русский": "ru"
+    "iso_code": "nl",
+    "self_name": "Nederlands",
+    "translation_count": 49
   },
   {
-    "Sardu": "sc"
+    "iso_code": "nqo",
+    "self_name": "ߒߞߏ",
+    "translation_count": 43
   },
   {
-    "slovenčina": "sk"
+    "iso_code": "or",
+    "self_name": "ଓଡ଼ିଆ",
+    "translation_count": 49
   },
   {
-    "سرائیکی": "skr-arab"
+    "iso_code": "pl",
+    "self_name": "Polski",
+    "translation_count": 31
   },
   {
-    "slovenščina": "sl"
+    "iso_code": "ru",
+    "self_name": "русский",
+    "translation_count": 57
   },
   {
-    "Shqip": "sq"
+    "iso_code": "sc",
+    "self_name": "Sardu",
+    "translation_count": 49
   },
   {
-    "Svenska": "sv"
+    "iso_code": "sk",
+    "self_name": "slovenčina",
+    "translation_count": 25
   },
   {
-    "ఇంగ్లీషు": "te"
+    "iso_code": "skr-arab",
+    "self_name": "سرائیکی",
+    "translation_count": 20
   },
   {
-    "Türkçe": "tr"
+    "iso_code": "sl",
+    "self_name": "slovenščina",
+    "translation_count": 57
   },
   {
-    "英语": "zh-hans"
+    "iso_code": "sq",
+    "self_name": "Shqip",
+    "translation_count": 49
   },
   {
-    "繁體中文": "zh-hant"
+    "iso_code": "sv",
+    "self_name": "Svenska",
+    "translation_count": 57
+  },
+  {
+    "iso_code": "te",
+    "self_name": "ఇంగ్లీషు",
+    "translation_count": 49
+  },
+  {
+    "iso_code": "tr",
+    "self_name": "Türkçe",
+    "translation_count": 57
+  },
+  {
+    "iso_code": "zh-hans",
+    "self_name": "英语",
+    "translation_count": 54
+  },
+  {
+    "iso_code": "zh-hant",
+    "self_name": "繁體中文",
+    "translation_count": 57
   }
 ])EXPECTEDRESPONSE");
 }
@@ -1161,7 +1329,6 @@ TEST_F(ServerTest, UserLanguageControl)
     const std::string description;
     const std::string url;
     const std::string acceptLanguageHeader;
-    const char* const requestCookie; // Cookie: header of the request
     const std::string expectedH1;
 
     operator TestContext() const
@@ -1172,64 +1339,45 @@ TEST_F(ServerTest, UserLanguageControl)
           {"acceptLanguageHeader", acceptLanguageHeader},
       };
 
-      if ( requestCookie ) {
-        ctx.push_back({"requestCookie", requestCookie});
-      }
-
       return ctx;
     }
   };
 
-  const char* const NO_COOKIE = nullptr;
-
   const TestData testData[] = {
     {
       "Default user language is English",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
       /*Accept-Language:*/ "",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "Not Found"
     },
     {
       "userlang URL query parameter is respected",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article?userlang=en",
       /*Accept-Language:*/ "",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "Not Found"
     },
     {
       "userlang URL query parameter is respected",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article?userlang=test",
       /*Accept-Language:*/ "",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "[I18N TESTING] Content not found, but at least the server is alive"
     },
     {
       "'Accept-Language: *' is handled",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
       /*Accept-Language:*/ "*",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "Not Found"
     },
     {
       "Accept-Language: header is respected",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
       /*Accept-Language:*/ "test",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "[I18N TESTING] Content not found, but at least the server is alive"
     },
-    {
-      "userlang cookie is ignored",
-      /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
-      /*Accept-Language:*/ "",
-      /*Request Cookie:*/       "userlang=test",
-      /* expected <h1> */ "Not Found"
-    },
     {
       "userlang query parameter takes precedence over Accept-Language",
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article?userlang=en",
       /*Accept-Language:*/ "test",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "Not Found"
     },
     {
@@ -1238,7 +1386,6 @@ TEST_F(ServerTest, UserLanguageControl)
       // with quality values) the most suitable language is selected.
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
       /*Accept-Language:*/ "test;q=0.9, en;q=0.2",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "[I18N TESTING] Content not found, but at least the server is alive"
     },
     {
@@ -1247,7 +1394,6 @@ TEST_F(ServerTest, UserLanguageControl)
       // with quality values) the most suitable language is selected.
       /*url*/ "/ROOT%23%3F/content/zimfile/invalid-article",
       /*Accept-Language:*/ "test;q=0.2, en;q=0.9",
-      /*Request Cookie:*/       NO_COOKIE,
       /* expected <h1> */ "Not Found"
     },
   };
@@ -1259,9 +1405,6 @@ TEST_F(ServerTest, UserLanguageControl)
     if ( !t.acceptLanguageHeader.empty() ) {
       headers.insert({"Accept-Language", t.acceptLanguageHeader});
     }
-    if ( t.requestCookie ) {
-      headers.insert({"Cookie", t.requestCookie});
-    }
     const auto r = zfs1_->GET(t.url.c_str(), headers);
     EXPECT_FALSE(r->has_header("Set-Cookie"));
     std::regex_search(r->body, h1Match, h1Regex);
@@ -1958,8 +2101,7 @@ TEST_F(ServerTest, viewerSettings)
 R"(const viewerSettings = {
   toolbarEnabled:       false,
   linkBlockingEnabled:  false,
-  libraryButtonEnabled: false,
-  defaultUserLanguage:  "en"
+  libraryButtonEnabled: false
 }
 )");
   }
@@ -1970,8 +2112,7 @@ R"(const viewerSettings = {
 R"(const viewerSettings = {
   toolbarEnabled:       false,
   linkBlockingEnabled:  true,
-  libraryButtonEnabled: false,
-  defaultUserLanguage:  "en"
+  libraryButtonEnabled: false
 }
 )");
   }
@@ -1982,8 +2123,7 @@ R"(const viewerSettings = {
 R"(const viewerSettings = {
   toolbarEnabled:       true,
   linkBlockingEnabled:  false,
-  libraryButtonEnabled: false,
-  defaultUserLanguage:  "en"
+  libraryButtonEnabled: false
 }
 )");
   }
@@ -1994,47 +2134,7 @@ R"(const viewerSettings = {
 R"(const viewerSettings = {
   toolbarEnabled:       true,
   linkBlockingEnabled:  false,
-  libraryButtonEnabled: true,
-  defaultUserLanguage:  "en"
-}
-)");
-  }
-
-  {
-    resetServer(ZimFileServer::WITH_TASKBAR_AND_LIBRARY_BUTTON);
-    const Headers headers{ {"Accept-Language", "fr"} };
-    ASSERT_EQ(zfs1_->GET("/ROOT%23%3F/viewer_settings.js", headers)->body,
-R"(const viewerSettings = {
-  toolbarEnabled:       true,
-  linkBlockingEnabled:  false,
-  libraryButtonEnabled: true,
-  defaultUserLanguage:  "fr"
-}
-)");
-  }
-
-  {
-    resetServer(ZimFileServer::WITH_TASKBAR_AND_LIBRARY_BUTTON);
-    const Headers headers{ {"Accept-Language", "test;q=0.2, en;q=0.9"} };
-    ASSERT_EQ(zfs1_->GET("/ROOT%23%3F/viewer_settings.js", headers)->body,
-R"(const viewerSettings = {
-  toolbarEnabled:       true,
-  linkBlockingEnabled:  false,
-  libraryButtonEnabled: true,
-  defaultUserLanguage:  "en"
-}
-)");
-  }
-
-  {
-    resetServer(ZimFileServer::WITH_TASKBAR_AND_LIBRARY_BUTTON);
-    const Headers headers{ {"Accept-Language", "test;q=0.9, en;q=0.2"} };
-    ASSERT_EQ(zfs1_->GET("/ROOT%23%3F/viewer_settings.js", headers)->body,
-R"(const viewerSettings = {
-  toolbarEnabled:       true,
-  linkBlockingEnabled:  false,
-  libraryButtonEnabled: true,
-  defaultUserLanguage:  "test"
+  libraryButtonEnabled: true
 }
 )");
   }
